@@ -90,7 +90,7 @@ sqlite3_stmt *ppStmtUpdateEvent;
 
 /**
  * \fn double atof_l(const char *nptr, const char *locale)
- * \brief Opens the connection to sqlite3 database
+ * \brief Locale independent atof
  * \param *nptr string to be converted
  * \param *locale locale used in conversion
  * \return String as numeric value
@@ -121,11 +121,11 @@ double atof_l(const char *nptr, const char *locale)
 }
 
 /**
- * \fn void db_setfilename(void)
- * \brief Opens the connection to sqlite3 database
+ * \fn void db_setfilename(char *fn)
+ * \brief Set the database filename
+ * \param *fn filename
  *
- * Opens the connection to sqlite3 database. If a database does not
- * exist, creates a new one. Leaves the database open.
+ * Sets the filename of current database
  *
  */
 void db_setfilename(char *fn)
@@ -140,6 +140,14 @@ void db_setfilename(char *fn)
   strncpy(result_db, fn, len+1);
 }
 
+/**
+ * \fn char *db_getfilename(void)
+ * \brief Gets the database filename
+ * \return A pointer to the filename
+ *
+ * Gets the filename of current database.
+ *
+ */
 char *db_getfilename(void)
 {
   return result_db;
@@ -190,7 +198,8 @@ void db_connect(void)
 				   "model TEXT,"
 				   "year NUMBER,"
 				   "register TEXT,"
-				   "notes TEXT);"
+				   "notes TEXT,"
+				   "fueltype INTEGER);"
 				   "CREATE TABLE record("
 				   "id INTEGER PRIMARY KEY AUTOINCREMENT,"
 				   "carid INTEGER,"
@@ -263,7 +272,7 @@ void db_connect(void)
 				   "km REAL);"
 				   );
 	if (SQLITE_OK != sqlite3_exec(db, create_sql, NULL, NULL, &zErrMsg) &&
-	    SQLITE_OK != sqlite3_get_table(db, "id FROM alarmtype LIMIT 1",&sqlresult, &sqlrows, &sqlcolumns, NULL)) {
+	    SQLITE_OK != sqlite3_get_table(db, "SELECT id FROM alarmtype LIMIT 1",&sqlresult, &sqlrows, &sqlcolumns, NULL)) {
 	  PDEBUG("Failed to open or create database\n");
 	  PDEBUG("%s\n",sqlite3_errmsg(db));
 	  sqlite3_close(db);
@@ -272,6 +281,26 @@ void db_connect(void)
       sqlite3_free(create_sql);
       sqlite3_free_table(sqlresult);
       }
+
+      /* Check if the database contains the fueltype column (0.7 and before did not) */
+      rc = sqlite3_get_table(db, "SELECT fueltype FROM car LIMIT 1",&sqlresult, &sqlrows, &sqlcolumns, NULL);
+      sqlite3_free_table(sqlresult);
+      if (rc != SQLITE_OK ) { /* create new column if not found */
+	PDEBUG("Creating new fueltype column to table car\n");
+	create_sql = sqlite3_mprintf(
+				     "ALTER TABLE car "
+				     "ADD COLUMN fueltype INTEGER;"
+				     );
+	if (SQLITE_OK != sqlite3_exec(db, create_sql, NULL, NULL, &zErrMsg) &&
+	    SQLITE_OK != sqlite3_get_table(db, "SELECT fueltype FROM car LIMIT 1",&sqlresult, &sqlrows, &sqlcolumns, NULL)) {
+	  PDEBUG("Failed to add fueltype column to database\n");
+	  PDEBUG("%s\n",sqlite3_errmsg(db));
+	  sqlite3_close(db);
+	  db = NULL;
+	}
+	sqlite3_free(create_sql);
+	sqlite3_free_table(sqlresult);
+      }
     }
 
     if (db) {
@@ -318,7 +347,7 @@ void db_connect(void)
       rc = sqlite3_prepare_v2(db, "SELECT mark,model,register,year FROM car where id=?;", -1, &ppStmtCurCar, NULL);
 
       /* prepare sql query for querying the current car */
-      rc = sqlite3_prepare_v2(db, "SELECT mark,model,year,register,notes FROM car where id=?;", -1, &ppStmtOneCar, NULL);
+      rc = sqlite3_prepare_v2(db, "SELECT mark,model,year,register,notes,fueltype FROM car where id=?;", -1, &ppStmtOneCar, NULL);
 
       /* prepare sql query for adding a new driver */
       rc = sqlite3_prepare_v2(db, "INSERT INTO driver(fullname,nickname) VALUES(?,?);", -1, &ppStmtAddDriver, NULL);
@@ -329,12 +358,12 @@ void db_connect(void)
 "SET fullname=?, nickname=? WHERE id=?;",-1, &ppStmtUpdateDriver, NULL);
 
       /* prepare sql query for adding a new car */
-      rc = sqlite3_prepare_v2(db, "INSERT INTO car(mark,model,year,register,notes) VALUES(?,?,?,?,?);", -1, &ppStmtAddCar, NULL);
+      rc = sqlite3_prepare_v2(db, "INSERT INTO car(mark,model,year,register,notes,fueltype) VALUES(?,?,?,?,?,?);", -1, &ppStmtAddCar, NULL);
 
       /* prepare sql query for changing car properties */
       rc = sqlite3_prepare_v2(db, 
 "UPDATE car "
-"SET mark=?, model=?, year=?, register=?, notes=? WHERE id=?;",-1, &ppStmtUpdateCar, NULL);
+"SET mark=?, model=?, year=?, register=?, notes=?, fueltype=? WHERE id=?;",-1, &ppStmtUpdateCar, NULL);
 
       /* prepare sql query for deleting a record */
       rc = sqlite3_prepare_v2(db, "DELETE FROM record WHERE id=?;", -1, &ppStmtDeleteRecord, NULL);
@@ -489,7 +518,7 @@ void db_close(void)
 
 /**
  * \fn int db_open(void)
- * \brief Tell if the database is open
+ * \brief Tells if the database is open
  * \return True if open, false if not
  */
 int db_open(void)
@@ -497,8 +526,80 @@ int db_open(void)
   return db != NULL;
 }
 
-char *fields[]={"day", "km", "trip", "fill", "consum", "price", "priceperlitre", 
-	     "service", "oil", "tires", "notes"};
+/**
+ * \fn float db_get_fueltype(int carid)
+ * \brief Finds the fueltype of car
+ * \return Fueltype of car (0 = petrol, 1 = diesel)
+ */
+float db_get_fueltype(int carid)
+{
+  char **sqlresult;
+  char *sqlquery;
+  int sqlrows;
+  int sqlcolumns;
+  int rc;
+  float ret;
+
+  ret=0.0;
+  if (db) {
+    sqlquery = sqlite3_mprintf("SELECT fueltype FROM car WHERE id=%d;",carid);
+    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+/*    sqlite3_free((void *)sqlquery); */
+    sqlite3_free(sqlquery);
+    if (rc==SQLITE_OK) {
+      if (sqlcolumns*(sqlrows+1)>0) {
+	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
+	  ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
+      }
+      sqlite3_free_table(sqlresult);
+    }
+  }
+
+  return ret;
+}
+
+/**
+ * \fn float get_emission_per_litre(int fueltype)
+ * \brief Returns the CO2 emission in g/l
+ * \param fueltype 0 for petrol, 1 for diesel
+ * \return CO2 emission in g/l
+ *
+ */
+float get_emission_per_litre(int fueltype)
+{
+  float emissionperlitre[]={2350.0, 2660.0};
+
+  return emissionperlitre[fueltype];
+
+}
+/**
+ * \fn float calc_co2_emission(float fill, float trip, int fueltype)
+ * \brief Returns the CO2 emission in g/km
+ * \param fill The amount of fuel filled
+ * \param trip The distance driven
+ * \param carid The car id
+ * \return CO2 emission in g/km
+ *
+ */
+float calc_co2_emission(float fill, float trip, int carid)
+{
+  float result;
+  int fueltype;
+
+  fueltype = db_get_fueltype(carid);
+  if (fueltype<0) fueltype=0;
+  if (fueltype>1) fueltype=1;
+
+  if (trip>1e-5)
+    result=get_emission_per_litre(fueltype)*fill/trip;
+  else
+    result=0.0;
+  return result;
+}
+
+char *fields[]={"day", "km", "trip", "fill", "consum", "price", 
+		"price/trip", "priceperlitre", 
+		"service", "oil", "tires", "fill/trip", "notes"};
 
 /**
  * \fn float db_get_max(int fieldnum)
@@ -514,6 +615,7 @@ float db_get_max(int fieldnum)
   int rc;
   float ret;
 
+  ret=0.0;
   if (fieldnum<sizeof(fields)/sizeof(char *)) {
     if (db) {
       sqlquery = sqlite3_mprintf("SELECT max(%s) FROM record WHERE carid=%d;",fields[fieldnum],currentcar);
@@ -524,37 +626,35 @@ float db_get_max(int fieldnum)
 	if (sqlcolumns*(sqlrows+1)>0) {
 	  if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
 	    ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
-	  else
-	    ret=0.0;
 	}
 	sqlite3_free_table(sqlresult);
       }
-      else
-	ret=0.0;
     }
-    else
-      ret=0.0;
   }
-  else
-    ret=0.0;
 
   return ret;
 }
 
 /**
- * \fn float db_get_max_date(char *date, int n)
+ * \fn void db_get_max_date(char **date)
  * \brief Finds the maximum value of date
- * \return Maximum value date
+ * \param **date buffer for the maximum value of date
+ *
+ * Finds the maximum of value dates in the database for
+ * currentdriver. The date buffer is allocated and must be freed after
+ * usage.
+ *
  */
-float db_get_max_date(char *date, int n)
+void db_get_max_date(char **date)
 {
   char **sqlresult;
   char *sqlquery;
   int sqlrows;
   int sqlcolumns;
   int rc;
-  float ret;
+  int len;
 
+  *date=NULL;
   if (db) {
     sqlquery = sqlite3_mprintf("SELECT max(day) FROM record WHERE carid=%d;",currentcar);
     rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -562,23 +662,21 @@ float db_get_max_date(char *date, int n)
     sqlite3_free(sqlquery);
     if (rc==SQLITE_OK) {
       if (sqlcolumns*(sqlrows+1)>0) {
-	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
-	  strncpy(date, sqlresult[sqlcolumns*(sqlrows+1)-1],n);
-	else
-	  date[0]='\0';
+	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) {
+	  len=strlen(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+	  *date=(char *)malloc((len+1)*sizeof(char));
+	  strncpy(*date, sqlresult[sqlcolumns*(sqlrows+1)-1],len+1);
+	}
       }
       sqlite3_free_table(sqlresult);
     }
-    else
-      date[0]='\0';
   }
-  else
-    date[0]='\0';
 }
 
 /**
  * \fn float db_get_min(int fieldnum)
  * \brief Finds the minimum value of given field
+ * \param fieldnum the field number
  * \return Minimum value of the given field
  */
 float db_get_min(int fieldnum)
@@ -590,6 +688,7 @@ float db_get_min(int fieldnum)
   int rc;
   float ret;
 
+  ret=0.0;
   if (fieldnum<sizeof(fields)/sizeof(char *)) {
     if (db) {
       sqlquery = sqlite3_mprintf("SELECT min(%s) FROM record WHERE carid=%d;",fields[fieldnum],currentcar);
@@ -600,37 +699,35 @@ float db_get_min(int fieldnum)
 	if (sqlcolumns*(sqlrows+1)>0) {
 	  if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
 	    ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
-	  else
-	    ret=0.0;
 	}
 	sqlite3_free_table(sqlresult);
       }
-      else
-	ret=0.0;
     }
-    else
-      ret=0.0;
   }
-  else
-    ret=0.0;
 
   return ret;
 }
 
 /**
- * \fn float db_get_min_date(char *date, int n)
- * \brief Finds the maximum value of date
- * \return Maximum value date
+ * \fn void db_get_min_date(char **date)
+ * \brief Finds the minimum value of date
+ * \param **date buffer for the maximum value of date
+ *
+ * Finds the minimum of value dates in the database for
+ * currentdriver. The date buffer is allocated and must be freed after
+ * usage.
+ *
  */
-float db_get_min_date(char *date, int n)
+void db_get_min_date(char **date)
 {
   char **sqlresult;
   char *sqlquery;
   int sqlrows;
   int sqlcolumns;
   int rc;
-  float ret;
+  int len;
 
+  *date=NULL;
   if (db) {
     sqlquery = sqlite3_mprintf("SELECT min(day) FROM record WHERE carid=%d;",currentcar);
     rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -638,18 +735,15 @@ float db_get_min_date(char *date, int n)
     sqlite3_free(sqlquery);
     if (rc==SQLITE_OK) {
       if (sqlcolumns*(sqlrows+1)>0) {
-	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
-	  strncpy(date, sqlresult[sqlcolumns*(sqlrows+1)-1],n);
-	else
-	  date[0]='\0';
+	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) {
+	  len=strlen(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+	  *date=(char *)malloc((len+1)*sizeof(char));
+	  strncpy(*date, sqlresult[sqlcolumns*(sqlrows+1)-1],len+1);
+	}
       }
       sqlite3_free_table(sqlresult);
     }
-    else
-      date[0]='\0';
   }
-  else
-    date[0]='\0';
 }
 
 /**
@@ -666,6 +760,7 @@ float db_get_last_km(void)
   int rc;
   float ret;
 
+  ret=0.0;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT max(km) FROM record WHERE carid=%d;",currentcar);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -675,16 +770,10 @@ float db_get_last_km(void)
      if (sqlcolumns*(sqlrows+1)>0) {
        if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
 	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
-       else
-	 ret=0.0;
      }
      sqlite3_free_table(sqlresult);
    }
-   else
-     ret=0.0;
  }
- else
-   ret=0.0;
 
  return ret;
 }
@@ -703,6 +792,7 @@ float db_get_last_refill(void)
   int rc;
   float ret;
 
+  ret=0.0;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT km FROM record WHERE carid=%d AND trip>0 ORDER BY km DESC LIMIT 1;",currentcar);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -712,29 +802,29 @@ float db_get_last_refill(void)
      if (sqlcolumns*(sqlrows+1)>0) {
        if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
 	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
-       else
-	 ret=0.0;
      }
      sqlite3_free_table(sqlresult);
    }
-   else
-     ret=0.0;
  }
- else
-   ret=0.0;
 
  return ret;
 }
 
 /**
- * \fn int db_get_monthly_data(int year, double *x, double *y)
+ * \fn int db_get_monthly_data(int year, double **x, double **fill, double **trip, double **consum, double **priceperlitre)
  * \brief Get the monthly data of a given year
  * \param year The year the data is requested
- * \param *x   Pointer to month data
- * \param *y   Pointer to consumption data
- * \return Last refill km in SI units
+ * \param **x   Pointer to month data
+ * \param **fill Pointer to monthly fill data
+ * \param **trip Pointer to monthly trip data
+ * \param **consum Pointer to monthly consumption data
+ * \param **priceperlitre Pointer to monthly priceperlitre data
+ * \return Number of sql rows obtained
+ *
+ * Tables are allocated in this function and must be freed after usage.
+ *
  */
-int db_get_monthly_data(int year, double *x, double *fill, double *trip, double *consum, double *priceperlitre)
+int db_get_monthly_data(int year, double **x, double **fill, double **trip, double **consum, double **priceperlitre)
 {
   int i;
   char **sqlresult;
@@ -744,6 +834,11 @@ int db_get_monthly_data(int year, double *x, double *fill, double *trip, double
   int rc;
   int resultsize=0;
 
+  *x=NULL;
+  *fill=NULL;
+  *trip=NULL;
+  *consum=NULL;
+  *priceperlitre=NULL;
   if (db) {
     sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT * FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar);
     rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -758,12 +853,17 @@ int db_get_monthly_data(int year, double *x, double *fill, double *trip, double
       if (rc==SQLITE_OK) {
 	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) {
 	  resultsize=sqlrows;
+	  *x=(double *)malloc((resultsize)*sizeof(double));
+	  *fill=(double *)malloc((resultsize)*sizeof(double));
+	  *trip=(double *)malloc((resultsize)*sizeof(double));
+	  *consum=(double *)malloc((resultsize)*sizeof(double));
+	  *priceperlitre=(double *)malloc((resultsize)*sizeof(double));
 	  for (i=1;i<=sqlrows;i++) {
-	    x[i-1]=atof_l(sqlresult[i*sqlcolumns],"C");
-	    fill[i-1]=atof_l(sqlresult[i*sqlcolumns+2],"C");
-	    trip[i-1]=atof_l(sqlresult[i*sqlcolumns+3],"C");
-	    consum[i-1]=atof_l(sqlresult[i*sqlcolumns+4],"C");
-	    priceperlitre[i-1]=atof_l(sqlresult[i*sqlcolumns+5],"C");
+	    (*x)[i-1]=atof_l(sqlresult[i*sqlcolumns],"C");
+	    (*fill)[i-1]=atof_l(sqlresult[i*sqlcolumns+2],"C");
+	    (*trip)[i-1]=atof_l(sqlresult[i*sqlcolumns+3],"C");
+	    (*consum)[i-1]=atof_l(sqlresult[i*sqlcolumns+4],"C");
+	    (*priceperlitre)[i-1]=atof_l(sqlresult[i*sqlcolumns+5],"C");
 	  }
 	}
 	else
@@ -798,6 +898,7 @@ float db_get_totalkm(void)
  int rc;
  float ret;
 
+ ret=0.0;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT max(km)-min(km) FROM record WHERE carid=%d;",currentcar);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -807,16 +908,10 @@ float db_get_totalkm(void)
      if (sqlcolumns*(sqlrows+1)>0) {
        if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
 	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
-       else
-	 ret=0.0;
      }
      sqlite3_free_table(sqlresult);
    }
-   else
-     ret=0.0;
  }
- else
-   ret=0.0;
 
  return ret;
 }
@@ -835,6 +930,7 @@ float db_get_totalcost(void)
  int rc;
  float ret;
 
+ ret=0.0;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT sum(price)+sum(service)+sum(oil)+sum(tires) FROM record WHERE carid=%d",currentcar);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -844,16 +940,10 @@ float db_get_totalcost(void)
      if (sqlcolumns*(sqlrows+1)>0) {
        if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
 	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
-       else
-	 ret=0.0;
      }
      sqlite3_free_table(sqlresult);
    }
-   else
-     ret=0.0;
  }
- else
-   ret=0.0;
 
  return ret;
 }
@@ -881,6 +971,7 @@ float db_get_totalfill(void)
  int rc;
  float ret;
 
+ ret=0.0;
  if (db) {
 /*    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT fill FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar); */
 /*    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL); */
@@ -931,20 +1022,117 @@ float db_get_totalfill(void)
      }
      else {
        sqlite3_free_table(sqlresult);
-       ret=0.0;
      }
    }
-   else
-     ret=0.0;
 
  }
- else
-   ret=0.0;
 
  return ret;
 }
 
+/**
+ * \fn int db_get_yearly_consum(int **year, int **numfills, float **minconsum, float **avgconsum, float **maxconsum)
+ * \brief Get the minimun, maximum and average consumption of currentcar
+ * \param **year The year output table
+ * \param **numfills How many fill events for given year
+ * \param **minconsum Minimum consumption in the given year
+ * \param **avgconsum Average consumption in the given year
+ * \param **maxconsum Maximum consumption in the given year
+ * \return Size of output tables
+ *
+ * Tables are allocated in this function and must be freed after usage.
+ *
+ */
+int db_get_yearly_consum(int **year, int **numfills, float **minconsum, float **avgconsum, float **maxconsum)
+{
+  char **sqlresult;
+  char *sqlquery;
+  int sqlrows;
+  int sqlcolumns;
+  int rc;
+  int i;
+  int resultsize=0;
 
+  *year=NULL;
+  *numfills=NULL;
+  *minconsum=NULL;
+  *avgconsum=NULL;
+  *maxconsum=NULL;
+  if (db) {
+    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT * FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar);
+    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+    sqlite3_free((void *)sqlquery);
+    sqlite3_free_table(sqlresult);
+    if (rc==SQLITE_OK) {
+      sqlquery = sqlite3_mprintf("SELECT STRFTIME(\"%s\",DATE(day)),COUNT(*),MIN(consum),SUM(fill)/SUM(trip)*100,MAX(consum) FROM fillview WHERE fill>0 GROUP BY STRFTIME(\"%s\",DATE(day))","%Y","%Y");
+      rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+      sqlite3_free((void *)sqlquery);
+      if (rc==SQLITE_OK) {
+	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) {
+	  resultsize=sqlrows;
+	  *year=(int *)malloc((resultsize)*sizeof(int));
+	  *numfills=(int *)malloc((resultsize)*sizeof(int));
+	  *minconsum=(float *)malloc((resultsize)*sizeof(float));
+	  *avgconsum=(float *)malloc((resultsize)*sizeof(float));
+	  *maxconsum=(float *)malloc((resultsize)*sizeof(float));
+	  for (i=1;i<=resultsize;i++) {
+	    (*year)[i-1]=(int)atof_l(sqlresult[i*sqlcolumns],"C");
+	    (*numfills)[i-1]=(int)atof_l(sqlresult[i*sqlcolumns+1],"C");
+	    (*minconsum)[i-1]=(float)atof_l(sqlresult[i*sqlcolumns+2],"C");
+	    (*avgconsum)[i-1]=(float)atof_l(sqlresult[i*sqlcolumns+3],"C");
+	    (*maxconsum)[i-1]=(float)atof_l(sqlresult[i*sqlcolumns+4],"C");
+	  } // for
+	}
+      } // if(rc==SQLITE_OK)
+      else
+	PDEBUG("No data found\n");
+
+      sqlite3_free_table(sqlresult);
+
+      rc = sqlite3_get_table(db, "DROP VIEW fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL);
+      if (rc==SQLITE_OK)
+	sqlite3_free_table(sqlresult);
+    } // if(rc==SQLITE_OK)
+
+  } // if (db)
+
+  return resultsize;
+}
+
+/**
+ * \fn int db_get_consum_bin(int year, float minconsum, float maxconsum)
+ * \brief Get the number of trip consumptions between minconsum and maxconsum
+ * \param year The year
+ * \param minconsum Lower limit of the bin
+ * \param maxconsum Higher limit of the bin
+ * \return Number of trip consumptions between minconsum and maxconsum
+ *
+ */
+int db_get_consum_bin(int year, float minconsum, float maxconsum)
+{
+  char **sqlresult;
+  char *sqlquery;
+  int sqlrows;
+  int sqlcolumns;
+  int rc;
+  int ret;
+
+  ret=0;
+  if (db) {
+    sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM record WHERE carid=%d AND STRFTIME(\"%s\",DATE(day))='%d' AND fill > 0 AND consum BETWEEN %f AND %f",currentcar,"%Y",year,minconsum,maxconsum);
+    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+    sqlite3_free((void *)sqlquery);
+    if (rc==SQLITE_OK) {
+      if (sqlcolumns*(sqlrows+1)>0) {
+	if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
+	  ret=atoi(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+      }
+      sqlite3_free_table(sqlresult);
+    } // if (rc==SQLITE_OK)
+  } // if (db)
+
+  return ret;
+}
 
 /**
  * \fn int db_get_last_insert_rowid(void)
@@ -979,56 +1167,55 @@ sqlite_int64 db_get_last_insert_rowid(void)
 }
 
 /**
- * \fn void db_get_driver(char *driver, int id, int n)
+ * \fn void db_get_driver(char **driver, int id)
  * \brief Get the name of the driver
- * \param *driver pointer the buffer for storing the name
+ * \param **driver pointer for storing the name
  * \param id the rowid of a record where the driver name is requested
- * \param n max length of the driver buffer
  *
  * This function returns the name of the driver of a given record. The
- * buffer to store the name must have been allocated before calling
- * this function.
+ * driver buffer is allocated and must be freed after usage.
  *
  */
-void db_get_driver(char *driver, int id, int n)
+void db_get_driver(char **driver, int id)
 {
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
+ int len;
 
+ *driver=NULL;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT nickname FROM driver WHERE id=%d",id);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
    if (rc==SQLITE_OK) {
-     if ( (sqlcolumns*(sqlrows+1))> 0)
-       strncpy(driver,sqlresult[sqlcolumns*(sqlrows+1)-1],n);
-     else
-       driver[0]='\0';
+     if ( (sqlcolumns*(sqlrows+1))> 0) {
+       len=strlen(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+       *driver=(char *)malloc((len+1)*sizeof(char));
+       strncpy(*driver,sqlresult[sqlcolumns*(sqlrows+1)-1],len+1);
+     }
      sqlite3_free_table(sqlresult);
    }
    sqlite3_free((void *)sqlquery);
  }
- else
-   strcpy(driver,"db closed");
+/*  else */
+/*    strcpy(driver,"db closed"); */
 }
 
 
 /**
- * \fn void db_get_current_driver(char *driver, int n)
+ * \fn void db_get_current_driver(char **driver)
  * \brief Get the name of current driver
- * \param *driver pointer the buffer for storing the name
- * \param n max length of the driver buffer
+ * \param **driver pointer the buffer for storing the name
  *
- * This function returns the name of the current driver. The buffer to
- * store the name must have been allocated before calling this
- * function.
+ * This function returns the name of the current driver. The driver
+ * buffer is allocated and must be freed after usage.
  *
  */
-void db_get_current_driver(char *driver, int n)
+void db_get_current_driver(char **driver)
 {
-  db_get_driver(driver, currentdriver, n);
+  db_get_driver(driver, currentdriver);
 }
 
 /**
@@ -1065,6 +1252,7 @@ int db_get_numof_drivers(void)
 /**
  * \fn int db_get_numof_records(int id)
  * \brief Get the number of records in the database with a given driver
+ * \param id driver id
  * \return The number of records in the database with a given driver
  *
  * This function returns the number of records in the database with a
@@ -1100,6 +1288,7 @@ int db_get_numof_records(int id)
 /**
  * \fn int db_get_numof_events(int id)
  * \brief Get the number of events in the database with a given driver
+ * \param id driver id
  * \return The number of events in the database with a given driver
  *
  * This function returns the number of events in the database with a 
@@ -1135,6 +1324,7 @@ int db_get_numof_events(int id)
 /**
  * \fn db_get_numof_events_with_record(int id)
  * \brief Get the number of events in the database with a corresponding record in the fuel database
+ * \param id record id
  * \return The number of events in the database with a given driver
  *
  * This function returns the number of events in the database for a 
@@ -1201,6 +1391,7 @@ int db_get_numof_cars(void)
 /**
  * \fn int db_get_numof_carrecords(int id)
  * \brief Get the number of records in the database for a given car
+ * \param id car id
  * \return The number of records in the database with a given driver
  *
  * This function returns the number of records in the database with a
@@ -1236,6 +1427,7 @@ int db_get_numof_carrecords(int id)
 /**
  * \fn int db_get_numof_carevents(int id)
  * \brief Get the number of events in the database with a given car
+ * \param id car id
  * \return The number of events in the database with a given driver
  *
  * This function returns the number of events in the database with a 
@@ -1271,6 +1463,7 @@ int db_get_numof_carevents(int id)
 /**
  * \fn int db_get_numof_alarmtypes(int id)
  * \brief Get the number of alarmtypes in the database with a given car
+ * \param id car id
  * \return The number of alarmtypes in the database with a given car
  *
  * This function returns the number of alarmtypes in the database with a 
@@ -1303,13 +1496,18 @@ int db_get_numof_alarmtypes(int id)
  return ret;
 }
 
+/**
+ * \fn int db_delete_driver(int id)
+ * \brief Deletes a driver
+ * \param id the id of the driver
+ * \return nonzero if succesful
+ *
+ * This function deletes a driver, id of which is given as a paremeter.
+ *
+ */
 int db_delete_driver(int id)
 {
- char **sqlresult;
  char *sqlquery;
- int sqlrows;
- int sqlcolumns;
- int rc;
  int ret;
 
  ret=0;
@@ -1334,13 +1532,18 @@ int db_delete_driver(int id)
  return ret;
 }
 
+/**
+ * \fn int db_delete_car(int id)
+ * \brief Deletes a car
+ * \param id the id of the car
+ * \return nonzero if succesful
+ *
+ * This function deletes a car, id of which is given as a paremeter.
+ *
+ */
 int db_delete_car(int id)
 {
- char **sqlresult;
  char *sqlquery;
- int sqlrows;
- int sqlcolumns;
- int rc;
  int ret;
 
  ret=0;
@@ -1366,55 +1569,54 @@ int db_delete_car(int id)
 }
 
 /**
- * \fn void db_get_current_car(char *car, int n)
+ * \fn void db_get_current_car(char **car)
  * \brief Get the name of current car
- * \param *car pointer the buffer for storing the name
- * \param n max length of the car buffer
+ * \param **car pointer the buffer for storing the name
  *
- * This function returns the name of the current car. The buffer to
- * store the name must have been allocated before calling this
- * function.
+ * This function returns the name of the current car. The car buffer
+ * is allocated and must be freed after usage.
  *
  */
-void db_get_current_car(char *car, int n)
+void db_get_current_car(char **car)
 {
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
+ int len;
 
+ *car=NULL;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT mark FROM car WHERE id=%d",currentcar);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
    if (rc==SQLITE_OK) {
-     if ( (sqlcolumns*(sqlrows+1))> 0)
-       strncpy(car,sqlresult[sqlcolumns*(sqlrows+1)-1],n);
-     else
-       car[0]='\0';
+     if ( (sqlcolumns*(sqlrows+1))> 0) {
+       len=strlen(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+       *car=(char *)malloc((len+1)*sizeof(char));
+       strncpy(*car,sqlresult[sqlcolumns*(sqlrows+1)-1],len+1);
+     }
      sqlite3_free_table(sqlresult);
    }
    sqlite3_free((void *)sqlquery);
  }
- else
-   strcpy(car,"db closed");
+/*  else */
+/*    strcpy(car,"db closed"); */
 }
 
 /**
- * \fn int db_estimate_date(float km, char *estdate, int n, char *lastdate, double interval)
+ * \fn int db_estimate_date(float km, char **estdate, char *lastdate, double interval)
  * \brief Estimate the date when the given km will be reached
  * \param km the given km
- * \param *estdate pointer the buffer for storing the date
- * \param n max length of the date buffer
+ * \param **estdate pointer the buffer for storing the date
  * \param *lastdate date of last event
  * \param interval interval between services in months
  *
- * This function estimates when the given km will be reached. The
- * buffer to store the date must have been allocated before calling
- * this function.
+ * This function estimates when the given km will be reached. The date
+ * buffer is allocated and must be freed after usage.
  *
  */
-int db_estimate_date(float km, char *estdate, int n, char *lastdate, double interval)
+int db_estimate_date(float km, char **estdate, char *lastdate, double interval)
 {
   char **sqlresult;
   char *sqlquery;
@@ -1422,7 +1624,9 @@ int db_estimate_date(float km, char *estdate, int n, char *lastdate, double inte
   int sqlcolumns;
   int rc;
   int ret;
+  int len;
 
+ *estdate=NULL;
   ret = 1;
   if (db) {
     sqlquery = sqlite3_mprintf("CREATE TEMP VIEW kmview AS SELECT * FROM record WHERE carid=%d ORDER BY km DESC LIMIT 5;",currentcar);
@@ -1438,14 +1642,14 @@ int db_estimate_date(float km, char *estdate, int n, char *lastdate, double inte
       sqlite3_free((void *)sqlquery);
       if (rc==SQLITE_OK) {
 	if ( (sqlcolumns*(sqlrows+1))> 0) {
-	  if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
-	    strncpy(estdate,sqlresult[sqlcolumns*(sqlrows+1)-1],n);
-	  else
-	    estdate[0]='\0';
+	  if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) {
+	    len=strlen(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+	    *estdate=(char *)malloc((len+1)*sizeof(char));
+	    strncpy(*estdate,sqlresult[sqlcolumns*(sqlrows+1)-1],len+1);
+	  }
 	}
 	else {
 	  PDEBUG("No data found\n");
-	  estdate[0]='\0';
 	  ret=0;
 	}
 	sqlite3_free_table(sqlresult);
@@ -1458,7 +1662,6 @@ int db_estimate_date(float km, char *estdate, int n, char *lastdate, double inte
       }
     }
     else {
-      estdate[0]='\0';
       ret=0;
     }
   }
@@ -1510,24 +1713,40 @@ float db_estimate_km(char *lastdate, double interval)
   return ret;
 }
 
-void db_get_last_event(int alarmid, char *day, double *km, int n)
+/**
+ * \fn void db_get_last_event(int alarmid, char **day, double *km)
+ * \brief Gets the last event of a reminder
+ * \param alarmid the id of the reminder
+ * \param **day date of last event
+ * \param *km overall distance of last event
+ * \return nonzero if succesful
+ *
+ * This function obtains the last event of a reminder. The event data
+ * is returned as a date and overall distance is parameters day and
+ * km. The size of the string for date is given as parameter n.
+ *
+ */
+void db_get_last_event(int alarmid, char **day, double *km)
 {
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
+ int len;
 
+ *day=NULL;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT day, km FROM alarmevent WHERE alarmid=%d ORDER BY km DESC LIMIT 1",alarmid);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
    if (rc==SQLITE_OK) {
      if ( (sqlcolumns*(sqlrows+1))> 0) {
-       strncpy(day,sqlresult[sqlcolumns*(sqlrows+1)-2],n);
+       len=strlen(sqlresult[sqlcolumns*(sqlrows+1)-2]);
+       *day=(char *)malloc((len+1)*sizeof(char));
+       strncpy(*day,sqlresult[sqlcolumns*(sqlrows+1)-2],len+1);
        *km=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
      }
      else {
-       day[0]='\0';
        *km=0.0;
      }
      sqlite3_free_table(sqlresult);
@@ -1535,38 +1754,58 @@ void db_get_last_event(int alarmid, char *day, double *km, int n)
    sqlite3_free((void *)sqlquery);
  }
  else {
-   strcpy(day,"db closed");
+   strcpy(*day,"db closed");
    *km=0.0;
  }
 }
 
-void db_calc_date(char *newdate, char *date, double num, char *unit, int n)
+/**
+ * \fn void db_calc_date(char **newdate, char *date, double num, char *unit)
+ * \brief 
+ * \param **newdate The calculated date
+ * \param *date The original date
+ * \param num How many units (days, months) to differ from date
+ * \param *unit The name of the unit (e.g. "months")
+ *
+ * Calculates a date which differes the given number of units from the
+ * original date.
+ *
+ */
+void db_calc_date(char **newdate, char *date, double num, char *unit)
 {
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
  int sqlcolumns;
  int rc;
+ int len;
 
+ *newdate=NULL;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT DATE(\"%s\",'%d'||'%s');",date,(int)num,unit);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
    if (rc==SQLITE_OK) {
      if ( (sqlcolumns*(sqlrows+1))> 0) {
-       strncpy(newdate,sqlresult[sqlcolumns*(sqlrows+1)-1],n);
-     }
-     else {
-       newdate[0]='\0';
+       len=strlen(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+       *newdate=(char *)malloc((len+1)*sizeof(char));
+       strncpy(*newdate,sqlresult[sqlcolumns*(sqlrows+1)-1],len+1);
      }
      sqlite3_free_table(sqlresult);
    }
    sqlite3_free((void *)sqlquery);
  }
- else {
-   strcpy(newdate,"db closed");
- }
+/*  else { */
+/*    strcpy(newdate,"db closed"); */
+/*  } */
 }
 
+/**
+ * \fn int db_events_exist(int alarmid)
+ * \brief Checks if the given reminder has events associated with it
+ * \param alarmid the id of the reminder
+ * \return nonzero if events exist, 0 if not
+ *
+ */
 int db_events_exist(int alarmid)
 {
  char **sqlresult;
@@ -1576,6 +1815,7 @@ int db_events_exist(int alarmid)
  int rc;
  int retval;
 
+ retval=0;
  if (db) {
    sqlquery = sqlite3_mprintf("SELECT id FROM alarmevent WHERE alarmid=%d LIMIT 1",alarmid);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -1583,19 +1823,21 @@ int db_events_exist(int alarmid)
      if ( (sqlcolumns*(sqlrows+1))> 0) {
        retval=1;
      }
-     else {
-       retval=0;
-     }
      sqlite3_free_table(sqlresult);
    }
    sqlite3_free((void *)sqlquery);
  }
- else {
-   retval=0;
- }
+
  return retval;
 }
 
+/**
+ * \fn int db_delete_events(int alarmid)
+ * \brief Deletes the events of a reminder
+ * \param alarmid the id of the reminder
+ * \return nonzero if succesful, 0 if not
+ *
+ */
 int db_delete_events(int alarmid)
 {
  char **sqlresult;
@@ -1605,6 +1847,7 @@ int db_delete_events(int alarmid)
  int rc;
  int retval;
 
+ retval=0;
  if (db) {
    sqlquery = sqlite3_mprintf("DELETE FROM alarmevent WHERE alarmid=%d",alarmid);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -1625,6 +1868,13 @@ int db_delete_events(int alarmid)
  return retval;
 }
 
+/**
+ * \fn int db_delete_reminder(int id)
+ * \brief Deletes a reminder
+ * \param id the id of the reminder
+ * \return nonzero if succesful, 0 if not
+ *
+ */
 int db_delete_reminder(int id)
 {
  char **sqlresult;
@@ -1634,6 +1884,7 @@ int db_delete_reminder(int id)
  int rc;
  int retval;
 
+ retval=0;
  if (db) {
    sqlquery = sqlite3_mprintf("DELETE FROM alarmtype WHERE id=%d",id);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
@@ -1641,16 +1892,11 @@ int db_delete_reminder(int id)
      if ( (sqlcolumns*(sqlrows+1))> 0) {
        retval=1;
      }
-     else {
-       retval=0;
-     }
      sqlite3_free_table(sqlresult);
    }
    sqlite3_free((void *)sqlquery);
  }
- else {
-   retval=0;
- }
+
  return retval;
 }
 
