@@ -186,18 +186,21 @@ sort_alarmiter_compare_func (GtkTreeModel *model,
       break;
     case ALM_COL_LASTDATE:
     case ALM_COL_NEXTDATE: {
-      gchar *date1, *date2;
+      char *date1, *date2;
+      gchar *userdate1, *userdate2;
 
-      gtk_tree_model_get(model, a, sortcol, &date1, -1);
-      gtk_tree_model_get(model, b, sortcol, &date2, -1);
+      gtk_tree_model_get(model, a, sortcol, &userdate1, -1);
+      gtk_tree_model_get(model, b, sortcol, &userdate2, -1);
 
-      date2sqlite(date1, ENTRYDATEMAX, datefmtstr[curunit.dateformat], date1);
-      date2sqlite(date2, ENTRYDATEMAX, datefmtstr[curunit.dateformat], date2);
+      date2sqlite(&date1, datefmtstr[curunit.dateformat], userdate1);
+      date2sqlite(&date2, datefmtstr[curunit.dateformat], userdate2);
 
       ret=strcmp(date1,date2);
 
-      g_free(date1);
-      g_free(date2);
+      free(date1);
+      free(date2);
+      g_free(userdate1);
+      g_free(userdate2);
 
     }
       break;
@@ -239,7 +242,8 @@ void newevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
   gint64 id;
   sqlite_int64 neweventid;
   sqlite_int64 recordid;
-  char date[ENTRYDATEMAX];
+  char *date;
+  char *viewdate;
 #if PLAINGTK == 0
   guint year;
   guint month;
@@ -263,9 +267,10 @@ void newevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 
 #if PLAINGTK == 0
        hildon_date_editor_get_date(HILDON_DATE_EDITOR(pui->entryeventdate), &year, &month, &day);
+       date=(char *)malloc(ENTRYDATEMAX*sizeof(char));
        g_snprintf(date, ENTRYDATEMAX, "%d-%02d-%02d",year, month, day);
 #else
-       date2sqlite((char *)date, ENTRYDATEMAX, datefmtstr[curunit.dateformat], 
+       date2sqlite(&date, datefmtstr[curunit.dateformat], 
 		   gtk_entry_get_text(GTK_ENTRY(pui->entryeventdate)));
 #endif
        km = user2SIlength(atof(gtk_entry_get_text(GTK_ENTRY(pui->entryeventkm))));
@@ -305,6 +310,7 @@ void newevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 	  /*     ui_show_banner("Problem saving result"); */
 	  PDEBUG("Problem saving result");
 	  sqlite3_reset(ppStmtAddRecord);
+	  free(date);
 	}
 	else { /* record succesfully inserted */
 
@@ -322,10 +328,11 @@ void newevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 	      || SQLITE_DONE != sqlite3_step(ppStmtAddEvent)) {
 	    /*     ui_show_banner("Problem saving result"); */
 	    PDEBUG("Problem saving result\n");
+	    free(date);
 	  }
 	  else { /* record succesfully inserted */
 	    neweventid = db_get_last_insert_rowid();
-	    convdate(date, ENTRYDATEMAX, datefmtstr[curunit.dateformat], "%Y-%m-%d", date);
+	    convdate(&viewdate, datefmtstr[curunit.dateformat], "%Y-%m-%d", date);
 
 	    /* Add to the event list, if open */
 /* 	    if (pui->eventview != NULL) { */
@@ -333,7 +340,7 @@ void newevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 	      store = (GtkListStore *)gtk_tree_view_get_model(GTK_TREE_VIEW(pui->eventview));
 	      gtk_list_store_append (store, &iter);
 	      gtk_list_store_set (store, &iter,
-				  EVENT_COL_DAY,date,
+				  EVENT_COL_DAY,viewdate,
 				  EVENT_COL_KM,SIlength2user(km),
 				  EVENT_COL_ID,neweventid,
 				  EVENT_COL_RECID,recordid,
@@ -344,7 +351,7 @@ void newevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 	    get_store_and_iter(NULL, pui->view, NULL, &store, NULL);
 	    gtk_list_store_append (store, &storeiter);
 	    gtk_list_store_set (store, &storeiter,
-				COL_DAY, date,
+				COL_DAY, viewdate,
 				COL_KM, SIlength2user(km),
 				COL_TRIP, 0.0,
 				COL_FILL, 0.0,
@@ -362,6 +369,8 @@ void newevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 				-1);
 	  
 	    update_totalkm(pui);
+	    free(viewdate);
+	    free(date);
 
 	  }
 	  sqlite3_reset(ppStmtAddEvent);
@@ -394,8 +403,8 @@ void editevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui,
   GtkTreeIter    iter;
   gint64 id;
   sqlite_int64 curid;
-  char date[ENTRYDATEMAX];
-  char viewdate[ENTRYDATEMAX];
+  char *date;
+  char *viewdate;
 #if PLAINGTK == 0
   guint year;
   guint month;
@@ -414,9 +423,10 @@ void editevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui,
 
 #if PLAINGTK == 0
        hildon_date_editor_get_date(HILDON_DATE_EDITOR(pui->entryeventdate), &year, &month, &day);
+       date=(char *)malloc(ENTRYDATEMAX*sizeof(char));
        g_snprintf(date, ENTRYDATEMAX, "%d-%02d-%02d",year, month, day);
 #else
-       date2sqlite((char *)date, ENTRYDATEMAX, datefmtstr[curunit.dateformat], 
+       date2sqlite(&date, datefmtstr[curunit.dateformat], 
 		   gtk_entry_get_text(GTK_ENTRY(pui->entryeventdate)));
 #endif
        km = user2SIlength(atof(gtk_entry_get_text(GTK_ENTRY(pui->entryeventkm))));
@@ -444,10 +454,11 @@ void editevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui,
 	   || SQLITE_DONE != sqlite3_step(ppStmtUpdateRecord)) {
 	 /*     ui_show_banner("Problem saving result"); */
 	 PDEBUG("Problem saving result");
+	 free(date);
        }
        else {
 	 /* Store to event view */
-	 convdate(viewdate, ENTRYDATEMAX, datefmtstr[curunit.dateformat], "%Y-%m-%d", date);
+	 convdate(&viewdate, datefmtstr[curunit.dateformat], "%Y-%m-%d", date);
 	 store = (GtkListStore *)gtk_tree_view_get_model(GTK_TREE_VIEW(pui->eventview));
 	 gtk_list_store_set (store, &iter,
 			     EVENT_COL_DAY,viewdate,
@@ -478,6 +489,8 @@ void editevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui,
 	 }
 	 else
 	   PDEBUG("Something wrong with finding the first iter\n");
+
+	 free(viewdate);
 	
        }
        sqlite3_reset(ppStmtUpdateRecord);
@@ -494,6 +507,7 @@ void editevent_response (GtkDialog *dialog, gint arg1, AppUIData *pui,
 	   || SQLITE_DONE != sqlite3_step(ppStmtUpdateEvent)) {
 	 PDEBUG("Problem updating event");
        }
+       free(date);
        sqlite3_reset(ppStmtUpdateEvent);
       }
       else {
@@ -530,7 +544,7 @@ create_and_fill_eventmodel (AppUIData *pui)
   GtkTreeSelection *selection;
   GtkTreeModel *model;
   gint64 id;
-  char date[ENTRYDATEMAX];
+  char *date;
 
   if (db_open()) {
     selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pui->alarmview));
@@ -547,7 +561,7 @@ create_and_fill_eventmodel (AppUIData *pui)
       if (SQLITE_OK == sqlite3_bind_int64(ppStmtGetEvents, 1, id)) {
 	while(SQLITE_ROW == sqlite3_step(ppStmtGetEvents)) {
 
-	  convdate(date, ENTRYDATEMAX, datefmtstr[curunit.dateformat], "%Y-%m-%d", sqlite3_column_text(ppStmtGetEvents,0));
+	  convdate(&date, datefmtstr[curunit.dateformat], "%Y-%m-%d", sqlite3_column_text(ppStmtGetEvents,0));
 
 	  gtk_list_store_append (store, &iter);
 	  gtk_list_store_set (store, &iter,
@@ -556,6 +570,8 @@ create_and_fill_eventmodel (AppUIData *pui)
 			      EVENT_COL_ID,sqlite3_column_int64(ppStmtGetEvents,2),
 			      EVENT_COL_RECID,sqlite3_column_int64(ppStmtGetEvents,3),
 			      -1);
+	  free(date);
+
 	}
       }
 
@@ -810,7 +826,7 @@ void callback_listevents( GtkAction * action, AppUIData *pui )
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin), 
 				 GTK_POLICY_NEVER, 
 				 GTK_POLICY_AUTOMATIC);
-  gtk_box_pack_start (GTK_BOX(vbox1), scrolledwin, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX(vbox1), scrolledwin, TRUE, TRUE, 0);
 
   pui->eventview = create_event_view_and_model (pui);
   g_signal_connect(pui->eventview, "row-activated",
@@ -853,7 +869,7 @@ void callback_listevents( GtkAction * action, AppUIData *pui )
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox1, label);
 
 #if PLAINGTK == 0
-  hildon_help_dialog_help_enable(GTK_DIALOG(dialog),
+  help_dialog_help_enable(GTK_DIALOG(dialog),
 				 HELP_ID_EDITEVENT,
 				 pui->app->osso);
 #endif
@@ -871,25 +887,18 @@ void create_event_editwin(AppUIData *pui, GtkWidget *dialog)
   GtkWidget *vbox;
   GtkWidget *table;
   GtkWidget *label;
-  GtkWidget *scrollwin;
   GtkTreeSelection *selection;
   GtkTreeIter iter;
   GtkTreeModel *model;
   gchar *desc;
 #if PLAINGTK == 1
-  char timebuf[TIMELENMAX];
+  char *timebuf;
 #endif
 
   vbox = GTK_WIDGET(GTK_DIALOG(dialog)->vbox);
 
-  scrollwin = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), 
-				 GTK_POLICY_NEVER, 
-				 GTK_POLICY_AUTOMATIC);
-  gtk_box_pack_start(GTK_BOX(vbox),scrollwin, TRUE, TRUE,0);
-
   table = gtk_table_new(4, 4, FALSE);
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollwin),table);
+  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 5);
 
   /* Header row */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pui->alarmview));
@@ -916,8 +925,9 @@ void create_event_editwin(AppUIData *pui, GtkWidget *dialog)
 #if PLAINGTK == 1
   pui->entryeventdate = gtk_entry_new(); 
   gtk_entry_set_max_length(GTK_ENTRY(pui->entryeventdate), ENTRYDATEMAX);
-  gettimefmt(timebuf, datefmtstr[curunit.dateformat]);
+  gettimefmt(&timebuf, datefmtstr[curunit.dateformat]);
   gtk_entry_set_text(GTK_ENTRY(pui->entryeventdate),timebuf);
+  free(timebuf);
 #else
   pui->entryeventdate = hildon_date_editor_new();
 #endif
@@ -1045,7 +1055,6 @@ void create_event_editwin(AppUIData *pui, GtkWidget *dialog)
 
 
   gtk_widget_show(table);
-  gtk_widget_show(scrollwin);
 }
 
 void callback_newevent( GtkAction * action, AppUIData *pui )
@@ -1078,7 +1087,7 @@ void callback_newevent( GtkAction * action, AppUIData *pui )
     }
 
 #if PLAINGTK == 0
-    hildon_help_dialog_help_enable(GTK_DIALOG(dialog),
+    help_dialog_help_enable(GTK_DIALOG(dialog),
 				   HELP_ID_ADDEVENT,
 				   pui->app->osso);
 #endif
@@ -1181,7 +1190,7 @@ void editevent(AppUIData *pui, GtkDialog *parent)
   GtkWidget *label;
   gint64 id;
   sqlite_int64 recordid;
-  char date[ENTRYDATEMAX];
+  char *date;
   char buf[BUFMAX];
 #if PLAINGTK == 0
   guint year;
@@ -1210,16 +1219,23 @@ void editevent(AppUIData *pui, GtkDialog *parent)
       if (SQLITE_OK == sqlite3_bind_int64(ppStmtGetOneEvent, 1, id)) {
 	while (SQLITE_ROW == sqlite3_step(ppStmtGetOneEvent)) {
 #if PLAINGTK == 1
-	  convdate(date, ENTRYDATEMAX, datefmtstr[curunit.dateformat], "%Y-%m-%d", sqlite3_column_text(ppStmtGetOneEvent,0));
+	  convdate(&date, datefmtstr[curunit.dateformat], "%Y-%m-%d", sqlite3_column_text(ppStmtGetOneEvent,0));
 
 	  gtk_entry_set_text(GTK_ENTRY(pui->entryeventdate),date);
+	  free(date);
 #else
-	  convdate(date, ENTRYDATEMAX, "%Y", "%Y-%m-%d", sqlite3_column_text(ppStmtGetOneEvent,0));
+	  convdate(&date, "%Y", "%Y-%m-%d", sqlite3_column_text(ppStmtGetOneEvent,0));
 	  year=atoi(date);
-	  convdate(date, ENTRYDATEMAX, "%m", "%Y-%m-%d", sqlite3_column_text(ppStmtGetOneEvent,0));
+	  free(date);
+
+	  convdate(&date, "%m", "%Y-%m-%d", sqlite3_column_text(ppStmtGetOneEvent,0));
 	  month=atoi(date);
-	  convdate(date, ENTRYDATEMAX, "%d", "%Y-%m-%d", sqlite3_column_text(ppStmtGetOneEvent,0));
+	  free(date);
+
+	  convdate(&date, "%d", "%Y-%m-%d", sqlite3_column_text(ppStmtGetOneEvent,0));
 	  day=atoi(date);
+	  free(date);
+
 	  hildon_date_editor_set_date(HILDON_DATE_EDITOR(pui->entryeventdate), year, month, day);
 #endif
 
@@ -1253,7 +1269,7 @@ void editevent(AppUIData *pui, GtkDialog *parent)
       sqlite3_reset(ppStmtOneRecord);
 
 #if PLAINGTK == 0
-      hildon_help_dialog_help_enable(GTK_DIALOG(dialog),
+      help_dialog_help_enable(GTK_DIALOG(dialog),
 				     HELP_ID_EDITEVENT,
 				     pui->app->osso);
 #endif
@@ -1279,7 +1295,7 @@ void editevent(AppUIData *pui, GtkDialog *parent)
     gtk_widget_show(label);
 
 #if PLAINGTK == 0
-    hildon_help_dialog_help_enable(GTK_DIALOG(dialog),
+    help_dialog_help_enable(GTK_DIALOG(dialog),
 				   HELP_ID_EDITEVENT,
 				   pui->app->osso);
 #endif
@@ -1310,7 +1326,7 @@ void update_alarm_headers(GtkWidget *view)
     gtk_tree_view_column_set_title(col, _("Last miles"));
 
   col = gtk_tree_view_get_column(GTK_TREE_VIEW(view), ALM_COL_NEXTKM);
-  if (curunit.volumeunit==SI)
+  if (curunit.lengthunit==SI)
     gtk_tree_view_column_set_title(col, _("Next km"));
   else
     gtk_tree_view_column_set_title(col, _("Next miles"));
@@ -1330,11 +1346,14 @@ void update_event_headers(GtkWidget *view)
 gboolean update_next_event(GtkTreeModel *store)
 {
   GtkTreeIter    iter;
-  gint id;
-  char curdate[ENTRYDATEMAX];
-  char date[ENTRYDATEMAX];
-  char estdate[ENTRYDATEMAX];
-  char estdays[ENTRYDATEMAX];
+  gint64 id;
+  char *curdate;
+  char *lastdate;
+  char *userlastdate;
+  char *estdate;
+  char *userestdate;
+  char warnbuf[ENTRYDATEMAX];
+  char *estdays;
   double km;
   double distance;
   double interval;
@@ -1348,36 +1367,42 @@ gboolean update_next_event(GtkTreeModel *store)
   anywarnings=FALSE;
   if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter)) {
     lastkm=db_get_last_km();
-    gettime(curdate);
+    gettime(&curdate);
     do {
       gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 
 			 ALM_COL_ID, &id,
 			 ALM_COL_KMLIM, &distance,
 			 ALM_COL_TIMELIM, &interval,
 			 -1);
-      db_get_last_event(id, date, &km,ENTRYDATEMAX);
+      distance=user2SIlength(distance);
+      db_get_last_event(id, &lastdate, &km);
       if (distance > 0.0) { /* Distance limit given */
-	db_estimate_date(km+distance,estdate,ENTRYDATEMAX,date,interval);
-	if (strlen(date)>0) {
+	db_estimate_date(km+distance,&estdate,lastdate,interval);
+	if (lastdate != NULL && strlen(lastdate)>0) {
 	  expired=strcmp(curdate,estdate) >= 0;
-	  
+
 	  /* Determine the day when warning given */
-	  convdate(estdays,ENTRYDATEMAX, "%s", "%Y-%m-%d",estdate);
+	  convdate(&estdays, "%s", "%Y-%m-%d",estdate);
 	  warnday=atof(estdays)-warndays*3600*24;
-	  g_snprintf(estdays, ENTRYDATEMAX, "%ld", warnday);
-	  convdate(estdays, ENTRYDATEMAX, "%Y-%m-%d", "%s", estdays);
+	  free(estdays);
+
+	  g_snprintf(warnbuf, ENTRYDATEMAX, "%ld", warnday);
+	  convdate(&estdays, "%Y-%m-%d", "%s", warnbuf);
 	  warn=(strcmp(curdate,estdays) >=0) || (lastkm+warnkm > km+distance);
+	  free(estdays);
 	  
-	  convdate(date, ENTRYDATEMAX, datefmtstr[curunit.dateformat], "%Y-%m-%d",date);
-	  convdate(estdate, ENTRYDATEMAX, datefmtstr[curunit.dateformat], "%Y-%m-%d",estdate);
+	  convdate(&userlastdate, datefmtstr[curunit.dateformat], "%Y-%m-%d",lastdate);
+	  convdate(&userestdate, datefmtstr[curunit.dateformat], "%Y-%m-%d",estdate);
 	  
 	  gtk_list_store_set(GTK_LIST_STORE(store), &iter,
 			     ALM_COL_LASTKM, SIlength2user(km),
-			     ALM_COL_LASTDATE, date,
+			     ALM_COL_LASTDATE, userlastdate,
 			     ALM_COL_NEXTKM, SIlength2user(km+distance),
-			     ALM_COL_NEXTDATE, estdate,
+			     ALM_COL_NEXTDATE, userestdate,
 			     ALM_COL_COLOR, expired ? "red" : warn ? "yellow" : "green",
 			     -1);
+	  free(userestdate);
+	  free(userlastdate);
 	  anywarnings = anywarnings || expired || warn;
 	}
 	else {
@@ -1389,30 +1414,37 @@ gboolean update_next_event(GtkTreeModel *store)
 			     ALM_COL_COLOR, NULL,
 			     -1);
 	}
+	free(estdate);
       } /* Distance limit given */
       else { /* Only time limit given */
-	if (strlen(date)>0) {
-	  db_calc_date(estdate, date, interval, "months", ENTRYDATEMAX);
+	if (lastdate != NULL && strlen(lastdate)>0) {
+	  db_calc_date(&estdate, lastdate, interval, "months");
 	  expired=strcmp(curdate,estdate) >= 0;
-	  estkm=db_estimate_km(date, interval);
+	  estkm=db_estimate_km(lastdate, interval);
 
 	  /* Determine the day when warning given */
-	  convdate(estdays,ENTRYDATEMAX, "%s", "%Y-%m-%d",estdate);
+	  convdate(&estdays, "%s", "%Y-%m-%d",estdate);
 	  warnday=atof(estdays)-warndays*3600*24;
-	  g_snprintf(estdays, ENTRYDATEMAX, "%ld", warnday);
-	  convdate(estdays, ENTRYDATEMAX, "%Y-%m-%d", "%s", estdays);
-	  warn=(strcmp(curdate,estdays) >=0);
+	  free(estdays);
 
-	  convdate(date, ENTRYDATEMAX, datefmtstr[curunit.dateformat], "%Y-%m-%d",date);
-	  convdate(estdate, ENTRYDATEMAX, datefmtstr[curunit.dateformat], "%Y-%m-%d",estdate);
+	  g_snprintf(warnbuf, ENTRYDATEMAX, "%ld", warnday);
+	  convdate(&estdays, "%Y-%m-%d", "%s", warnbuf);
+	  warn=(strcmp(curdate,estdays) >=0);
+	  free(estdays);
+
+	  convdate(&userlastdate, datefmtstr[curunit.dateformat], "%Y-%m-%d",lastdate);
+	  convdate(&userestdate, datefmtstr[curunit.dateformat], "%Y-%m-%d",estdate);
 
 	  gtk_list_store_set(GTK_LIST_STORE(store), &iter,
 			     ALM_COL_LASTKM, SIlength2user(km),
-			     ALM_COL_LASTDATE, date,
-			     ALM_COL_NEXTKM, estkm,
-			     ALM_COL_NEXTDATE, estdate,
+			     ALM_COL_LASTDATE, userlastdate,
+			     ALM_COL_NEXTKM, SIlength2user(estkm),
+			     ALM_COL_NEXTDATE, userestdate,
 			     ALM_COL_COLOR, expired ? "red" : warn ? "yellow" : "green",
 			     -1);
+	  free(userestdate);
+	  free(userlastdate);
+	  free(estdate);
 	  anywarnings = anywarnings || expired || warn;
 
 	}
@@ -1426,7 +1458,9 @@ gboolean update_next_event(GtkTreeModel *store)
 			     -1);
 	}
       }
+      free(lastdate);
     } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter));
+    free(curdate);
   }
 
   return anywarnings;
@@ -1548,9 +1582,7 @@ GtkWidget *create_alarmview_and_model (AppUIData *pui)
   static data_func_t dfdata2;
   static data_func_t dfdata3;
   static data_func_t dfdata4;
-  static data_func_t dfdata5;
   static data_func_t dfdata6;
-  static data_func_t dfdata7;
 
   GtkTreeViewColumn   *col;
   GtkCellRenderer     *renderer;
@@ -1654,9 +1686,9 @@ GtkWidget *create_alarmview_and_model (AppUIData *pui)
   gtk_tree_view_column_add_attribute(col, renderer, "background", ALM_COL_COLOR);
 
   gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(col), ALM_COL_NEXTKM);
-  dfdata6.col = ALM_COL_NEXTKM;
-  strncpy(dfdata6.format,"%.0f",DFFORMATLEN);
-  gtk_tree_view_column_set_cell_data_func(col, renderer, cell_data_func, &dfdata6, NULL);
+  dfdata4.col = ALM_COL_NEXTKM;
+  strncpy(dfdata4.format,"%.0f",DFFORMATLEN);
+  gtk_tree_view_column_set_cell_data_func(col, renderer, cell_data_func, &dfdata4, NULL);
 
   /* --- Column #5 --- */
 
@@ -1700,9 +1732,9 @@ GtkWidget *create_alarmview_and_model (AppUIData *pui)
 
   gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(col), ALM_COL_LASTKM);
 
-  dfdata4.col = ALM_COL_LASTKM;
-  strncpy(dfdata4.format,"%.0f",DFFORMATLEN);
-  gtk_tree_view_column_set_cell_data_func(col, renderer, cell_data_func, &dfdata4, NULL);
+  dfdata6.col = ALM_COL_LASTKM;
+  strncpy(dfdata6.format,"%.0f",DFFORMATLEN);
+  gtk_tree_view_column_set_cell_data_func(col, renderer, cell_data_func, &dfdata6, NULL);
 
   /* --- Column #6 --- */
 
@@ -1812,16 +1844,20 @@ void newalarm_add(AppUIData *pui)
   GtkTreeSelection *selection;
   GtkListStore  *store;
   GtkTreeIter    iter;
+  gdouble km;
+  gdouble interval;
 
+  km = user2SIlength(atof(gtk_entry_get_text(GTK_ENTRY(pui->entryalarmdistance))));
+  interval = atof(gtk_entry_get_text(GTK_ENTRY(pui->entryalarminterval)));
   /* Read dialog entries and store to database */ 
   if (SQLITE_OK != sqlite3_bind_int(ppStmtAddAlarmtype, 1, currentcar) /* carid */
       || SQLITE_OK != sqlite3_bind_text(ppStmtAddAlarmtype, 2,
 					gtk_entry_get_text(GTK_ENTRY(pui->entryalarmdesc)),
 					-1, SQLITE_STATIC) /* shotdesc */
       || SQLITE_OK != sqlite3_bind_double(ppStmtAddAlarmtype, 3, 
-					  atof(gtk_entry_get_text(GTK_ENTRY(pui->entryalarmdistance)))) /* distance */
+					  km) /* distance */
       || SQLITE_OK != sqlite3_bind_double(ppStmtAddAlarmtype, 4,
-					  atof(gtk_entry_get_text(GTK_ENTRY(pui->entryalarminterval))))
+					  interval)
       || SQLITE_OK != sqlite3_bind_text(ppStmtAddAlarmtype, 5,
 					gtk_entry_get_text(GTK_ENTRY(pui->entryalarmlongdesc)),
 					-1, SQLITE_STATIC)
@@ -1833,8 +1869,8 @@ void newalarm_add(AppUIData *pui)
     gtk_list_store_append (store, &iter);
     gtk_list_store_set (store, &iter,
 			ALM_COL_DESC, gtk_entry_get_text(GTK_ENTRY(pui->entryalarmdesc)),
-			ALM_COL_KMLIM, user2SIlength(atof(gtk_entry_get_text(GTK_ENTRY(pui->entryalarmdistance)))),
-			ALM_COL_TIMELIM, atof(gtk_entry_get_text(GTK_ENTRY(pui->entryalarminterval))),
+			ALM_COL_KMLIM, SIlength2user(km),
+			ALM_COL_TIMELIM, interval,
 			ALM_COL_LASTKM, 0.0,
 			ALM_COL_LASTDATE, "",
 			ALM_COL_NEXTKM, 0.0,
@@ -1868,8 +1904,10 @@ void create_alarm_editwin(AppUIData *pui, GtkWidget *box, gboolean add)
   GtkWidget *scrollwin;
   gint row;
   gint64 id;
+  char buf[BUFMAX];
 
   scrollwin = gtk_scrolled_window_new(NULL, NULL);
+  gtk_widget_set_size_request(scrollwin, -1, DIALOG_MIN_HEIGHT1);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin), 
 				 GTK_POLICY_NEVER, 
 				 GTK_POLICY_AUTOMATIC);
@@ -1996,8 +2034,8 @@ void create_alarm_editwin(AppUIData *pui, GtkWidget *box, gboolean add)
       while (SQLITE_ROW == sqlite3_step(ppStmtGetOneAlarmtype)) {
 	gtk_entry_set_text(GTK_ENTRY(pui->entryalarmdesc), 
 			   sqlite3_column_text(ppStmtGetOneAlarmtype,0));
-	gtk_entry_set_text(GTK_ENTRY(pui->entryalarmdistance), 
-			   sqlite3_column_text(ppStmtGetOneAlarmtype,1));
+	g_snprintf(buf,BUFMAX,"%.0f",SIlength2user(sqlite3_column_double(ppStmtGetOneAlarmtype,1)));
+	gtk_entry_set_text(GTK_ENTRY(pui->entryalarmdistance), buf);
 	gtk_entry_set_text(GTK_ENTRY(pui->entryalarminterval), 
 			   sqlite3_column_text(ppStmtGetOneAlarmtype,2));
       }
@@ -2023,7 +2061,7 @@ GtkResponseType callback_newalarm( GtkAction * action, AppUIData *pui )
   create_alarm_editwin(pui, GTK_WIDGET(GTK_DIALOG(dialog)->vbox), TRUE);
 
 #if PLAINGTK == 0
-  hildon_help_dialog_help_enable(GTK_DIALOG(dialog),
+  help_dialog_help_enable(GTK_DIALOG(dialog),
 				 HELP_ID_ADDREMINDER,
 				 pui->app->osso);
 #endif
@@ -2112,7 +2150,7 @@ void callback_editalarm( GtkAction * action, AppUIData *pui )
     create_alarm_editwin(pui, GTK_WIDGET(GTK_DIALOG(dialog)->vbox), FALSE);
 
 #if PLAINGTK == 0
-    hildon_help_dialog_help_enable(GTK_DIALOG(dialog),
+    help_dialog_help_enable(GTK_DIALOG(dialog),
 				   HELP_ID_EDITREMINDER,
 				   pui->app->osso);
 #endif
