@@ -114,7 +114,7 @@ void callback_startimport(GtkDialog *widget, gint arg1, AppUIData *pui)
   gint fl;
   gboolean clocale;
   gint coldes[IMPORTCOLS];
-  char date[ENTRYDATEMAX];
+  char *date;
   gdouble km;
   gdouble trip;
   gdouble fill;
@@ -149,7 +149,7 @@ void callback_startimport(GtkDialog *widget, gint arg1, AppUIData *pui)
 
     for (i=fl;i<=plist.nodes;i++) {
 
-      date2sqlite(date, ENTRYDATEMAX, gtk_entry_get_text(GTK_ENTRY(pui->dateformatentry)), (char *)(NONNULLOREMPTY((char *)(csvgetrowcol(&plist, i, coldes[0])))));
+      date2sqlite(&date, gtk_entry_get_text(GTK_ENTRY(pui->dateformatentry)), (char *)(NONNULLOREMPTY((char *)(csvgetrowcol(&plist, i, coldes[0])))));
       km = user2SIlength(atof(NONNULLORZERO((char *)(csvgetrowcol(&plist, i, coldes[1])))));
       trip = user2SIlength(atof(NONNULLORZERO((char *)(csvgetrowcol(&plist, i, coldes[2])))));
       fill = user2SIvolume(atof(NONNULLORZERO((char *)(csvgetrowcol(&plist, i, coldes[3])))));
@@ -199,6 +199,8 @@ void callback_startimport(GtkDialog *widget, gint arg1, AppUIData *pui)
 	PDEBUG("Can't save new record - no database open");
       }
 
+      free(date);
+
     }
     update_record_model(pui);
     if (clocale)
@@ -496,7 +498,7 @@ void callback_import( GtkAction * action, AppUIData *pui )
     gtk_widget_show (pui->localecombo);
 
 #if PLAINGTK == 0
-    hildon_help_dialog_help_enable(GTK_DIALOG(importwindow),
+    help_dialog_help_enable(GTK_DIALOG(importwindow),
 				   HELP_ID_DATABASE,
 				   pui->app->osso);
 #endif
@@ -525,7 +527,9 @@ void callback_import( GtkAction * action, AppUIData *pui )
 
 void callback_export( GtkAction * action, gpointer data )
 {
-  char nickname[NICKMAXLEN];
+  double fill;
+  double trip;
+  char *nickname;
   gchar *filename = NULL;
   filename = interface_file_chooser ("Save file",
 				     GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -545,7 +549,7 @@ void callback_export( GtkAction * action, gpointer data )
 	  PDEBUG("Problem exporting");
 	}
 	else {
-	  fprintf(f,"\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\"\n",
+	  fprintf(f,"\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\"\n",
 		  sqlite3_column_text(ppStmtExportCar,0),
 		  sqlite3_column_text(ppStmtExportCar,1),
 		  sqlite3_column_text(ppStmtExportCar,2),
@@ -557,11 +561,12 @@ void callback_export( GtkAction * action, gpointer data )
 		  "",
 		  "",
 		  "",
+		  "",
 		  ""
 		  );
 	  sqlite3_reset(ppStmtExportCar);
 
-	  fprintf(f,"\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\"\n",
+	  fprintf(f,"\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\"\n",
 		  _("Date"),
 		  ( (curunit.lengthunit==SI)? _("Km") : _("Miles") ),
 		  ( (curunit.lengthunit==SI)? _("Trip [km]") : _("Trip [miles]") ),
@@ -572,28 +577,37 @@ void callback_export( GtkAction * action, gpointer data )
 		  _("Service"),
 		  _("Oil"),
 		  _("Tires"),
+		  ( (curunit.massunit==SI)? 
+		        (curunit.lengthunit==SI)? _("CO2 Emissions [g/km]") : _("CO2 Emissions [g/miles]") 
+		    : (curunit.lengthunit==SI)? _("CO2 Emissions [lb/100 km]") : _("CO2 Emissions [lb/100 miles]")),
 		  _("Notes"),
 		  _("Driver")
 		  );
 	  if (SQLITE_OK == sqlite3_bind_int(ppStmtExport, 1, currentcar)) {
 	    while(SQLITE_ROW == sqlite3_step(ppStmtExport)) {
 	    
-	      db_get_driver(nickname, sqlite3_column_int(ppStmtExport,11), NICKMAXLEN);
+	      db_get_driver(&nickname, sqlite3_column_int(ppStmtExport,11));
 
-	      fprintf(f,"\"%s\",\"%.2f\",\"%.2f\",\"%.2f\",\"%.2f\",\"%.2f\",\"%.3f\",\"%.2f\",\"%.2f\",\"%.2f\",\"%s\",\"%s\"\n",
+	      trip=sqlite3_column_double(ppStmtExport,2);
+	      fill=sqlite3_column_double(ppStmtExport,3);
+
+	      fprintf(f,"\"%s\",\"%.2f\",\"%.2f\",\"%.2f\",\"%.2f\",\"%.2f\",\"%.3f\",\"%.2f\",\"%.2f\",\"%.2f\",\"%.0f\",\"%s\",\"%s\"\n",
 		      sqlite3_column_text(ppStmtExport,0),
 		      SIlength2user(sqlite3_column_double(ppStmtExport,1)),
-		      SIlength2user(sqlite3_column_double(ppStmtExport,2)),
-		      SIvolume2user(sqlite3_column_double(ppStmtExport,3)),
+		      SIlength2user(trip),
+		      SIvolume2user(fill),
 		      SIconsumption2user(sqlite3_column_double(ppStmtExport,4)),
 		      sqlite3_column_double(ppStmtExport,5),
 		      SIppl2user(sqlite3_column_double(ppStmtExport,6)),
 		      sqlite3_column_double(ppStmtExport,7),
 		      sqlite3_column_double(ppStmtExport,8),
 		      sqlite3_column_double(ppStmtExport,9),
+		      SIemission2user(calc_co2_emission(fill,trip,currentcar)),
 		      sqlite3_column_text(ppStmtExport,10),
 		      nickname
 		      );
+	      if (nickname != NULL)
+		free(nickname);
 	    }
 	  }
 	}
