@@ -26,7 +26,7 @@
 #define HEADERLINE 0.1
 #define LEFTMARGIN 0.15
 #define INDENT 0.05
-#define FIRSTCOLUMN 0.4
+#define FIRSTCOLUMN 0.45
 #define COLUMNSEP 0.1
 #define HEADING1SIZE 16
 #define HEADING2SIZE 14
@@ -34,6 +34,8 @@
 
 /* These tags are used to identify texts that need to be changed
    according to the selected units */
+#define CO2TAG "CO2TAG"
+#define TOTALCO2TAG "TOTALCO2TAG"
 #define CONSUMETAG "CONSUMETAG"
 #define COSTPERKMTAG "COSTPERKMTAG"
 #define COSTTAG "COSTTAG"
@@ -502,9 +504,9 @@ static void create_report_toolbar(HildonWindow *window, GtkWidget *canvas)
  * \param *s The text to be changed
  * \return Text changed to take the unit account
  *
- * The text to be changed must be one of CONSUMETAG, COSTPERKMTAG,
- * COSTTAG OR PRICEPERUNITTAG. These are changed so that the selected
- * unit is taken into account.
+ * The text to be changed must be one of TOTALCO2TAG, CO2TAG,
+ * CONSUMETAG, COSTPERKMTAG, COSTTAG OR PRICEPERUNITTAG. These are
+ * changed so that the selected unit is taken into account.
  *
  */
 gchar *report_conv_cost_label(const char *s)
@@ -521,6 +523,24 @@ gchar *report_conv_cost_label(const char *s)
   else
       return g_strdup(_("Avg. miles/gallon"));
   }
+  if (strncmp(CO2TAG,s,strlen(CO2TAG))==0) {
+    if (curunit.massunit==SI) {
+      if (curunit.lengthunit==SI)
+	return g_strconcat(_("Avg. CO2"), " ", (curunit.lengthunit==SI)? 
+			   _("g/km") : _("g/mile"), NULL);
+    }
+    else
+      return g_strconcat(_("Avg. CO2"), " ", (curunit.lengthunit==SI)? 
+			 _("lb/100 km") : _("lb/100 miles"), NULL);
+  }
+  if (strncmp(TOTALCO2TAG,s,strlen(TOTALCO2TAG))==0) {
+    if (curunit.massunit==SI) {
+      if (curunit.lengthunit==SI)
+	return g_strconcat(_("Tot. CO2"), " ",  _("kg"), NULL);
+    }
+    else
+      return g_strconcat(_("Tot. CO2"), " ",_("lb"), NULL);
+  }
   if (strncmp(COSTTAG,s,strlen(COSTTAG))==0)
     return g_strjoin(" ",_("Cost"), curunit.currency->str, NULL);
   if (strncmp(COSTPERKMTAG,s,strlen(COSTPERKMTAG))==0)
@@ -553,10 +573,12 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 			gettext_noop("Beginning of year"), 
 			gettext_noop("End of year"), 
 			gettext_noop("Overall distance")};
-  gchar *fueltexts[] = {gettext_noop("Fuel"), 
+  gchar *fueltexts[] = {gettext_noop("Fuel and emissions"), 
 			  gettext_noop("Total consumption"), 
 			  COSTTAG, 
 			  CONSUMETAG,
+			  CO2TAG,
+			  TOTALCO2TAG,
 			  PRICEPERUNITTAG,
 			  COSTPERKMTAG};
   gchar *oiltexts[] = {gettext_noop("Oil"),
@@ -582,7 +604,9 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
   gchar sumfill[ENTRYKMMAX]; 
   gchar sumprice[ENTRYKMMAX];
   gchar avgconsum[ENTRYKMMAX];
+  gchar avgco2[ENTRYKMMAX];
   gchar avgprice[ENTRYKMMAX];
+  gchar totalco2[ENTRYKMMAX];
   gchar pricekm[ENTRYKMMAX];
   gchar priceoil[ENTRYKMMAX];
   gchar priceoilkm[ENTRYKMMAX];
@@ -824,6 +848,12 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 		    sqlite3_column_double(ppStmtGetReport,5));
 	 g_snprintf(avgconsum, ENTRYKMMAX, "%.1f",
 		    SIconsumption2user(sqlite3_column_double(ppStmtGetReport,6)));
+	 g_snprintf(avgco2, ENTRYKMMAX, "%.0f",
+		    SIemission2user(sqlite3_column_double(ppStmtGetReport,6)*
+				    get_emission_per_litre(db_get_fueltype(currentcar))/100));
+	 g_snprintf(totalco2, ENTRYKMMAX, "%.0f",
+		    SIemission2user(sqlite3_column_double(ppStmtGetReport,4)*
+				    get_emission_per_litre(db_get_fueltype(currentcar))/1000));
 	 g_snprintf(avgprice, ENTRYKMMAX, "%.2f",
 		    SIppl2user(sqlite3_column_double(ppStmtGetReport,7)));
 	 g_snprintf(pricekm, ENTRYKMMAX, "%.2f",
@@ -875,6 +905,12 @@ void report_response (GtkDialog *dialog, gint arg1, AppUIData *pui)
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
 			 GTK_JUSTIFY_LEFT, avgconsum);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
+	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
+			 GTK_JUSTIFY_LEFT, avgco2);
+	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
+	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
+			 GTK_JUSTIFY_LEFT, totalco2);
+	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
 	 report_put_text(pui->reportcanvas, textx, texty, "Helvetica", NORMALSIZE, 
 			 GTK_JUSTIFY_LEFT, avgprice);
 	 texty=report_line_down(texty, NORMALSIZE, 1.2, page_height);
@@ -1037,7 +1073,7 @@ void callback_report( GtkAction * action, AppUIData *pui )
   }
 
 #if PLAINGTK == 0
-  hildon_help_dialog_help_enable(GTK_DIALOG(dialog),
+  help_dialog_help_enable(GTK_DIALOG(dialog),
 				 HELP_ID_REPORT,
 				 pui->app->osso);
 #endif
