@@ -32,7 +32,7 @@
 #define XNOFULL_PREV .05
 #define YNOFULL_PREV .95
 
-enum analysis {CURRENT, CONSUM, FILL, TRIP, PRICEPERLITRE};
+enum analysis {CURRENT, CONSUM, FILL, TRIP, PRICEPERLITRE, CONSUMDISTR};
 
 int currentyear;
 
@@ -44,6 +44,7 @@ void plot_set_titles( AppUIData *pui, enum analysis method);
 void cb_stat( GtkAction * action, AppUIData *pui );
 void cb_trip( GtkAction * action, AppUIData *pui );
 void cb_priceperlitre( GtkAction * action, AppUIData *pui );
+void cb_consumdistr( GtkAction * action, AppUIData *pui );
 
 /**
  * \fn gboolean delete_statwin( GtkWidget *widget, GdkEvent  *event, gpointer   data )
@@ -88,16 +89,32 @@ plot_monthly_data(AppUIData *pui, enum analysis method)
 
   char yearbuf[YEARBUFLEN];
   GdkColor color;
-  static double px2[12] = {0.0};
-  static double fill[12] = {0.0};
-  static double trip[12] = {0.0};
-  static double consum[12] = {0.0};
-  static double priceperlitre[12] = {0.0};
+  static double *px2=NULL;
+  static double *fill=NULL;
+  static double *trip=NULL;
+  static double *consum=NULL;
+  static double *priceperlitre=NULL;
   double *yval;
   double ymax = 0.0;
   int nummonths;
   int i;
 
+  int numbins;
+  double binstart;
+  double binend;
+  double bindelta;
+  double binc;
+  double offset;
+  double bintemp;
+  static double *bin=NULL;
+  int *year;
+  int *numfills;
+  float *minconsum;
+  float *avgconsum;
+  float *maxconsum;
+  int numyears;
+
+
   static enum analysis currentmethod=0;
 
   if (method != CURRENT) {
@@ -114,53 +131,126 @@ plot_monthly_data(AppUIData *pui, enum analysis method)
   gtk_plot_add_data(GTK_PLOT(pui->active_plot), dataset);
   gtk_widget_show(GTK_WIDGET(dataset));
 
-  for (i=0;i<12;i++) {
-    fill[i]=0.0;
-    trip[i]=0.0;
-    consum[i]=0.0;
-    priceperlitre[i]=0.0;
-    px2[i]=i+1;
+  if (fill!=NULL) {
+    free(fill);
+    fill=NULL;
+  }
+  if (trip!=NULL) {
+    free(trip);
+    trip=NULL;
+  }
+  if (consum!=NULL) {
+    free(consum);
+    consum=NULL;
+  }
+  if (priceperlitre!=NULL) {
+    free(priceperlitre);
+    priceperlitre=NULL;
+  }
+  if (px2!=NULL) {
+    free(px2);
+    px2=NULL;
+  }
+  if (bin!=NULL) {
+    free(bin);
+    bin=NULL;
   }
 
   if (currentyear==0)
     currentyear=getyear();
-  nummonths=db_get_monthly_data(currentyear,px2,fill,trip,consum,priceperlitre);
-  for (i=0;i<12;i++) {
-    switch (currentmethod) {
-    case CONSUM:
-      yval=consum;
-      if (curunit.consumeunit != SI) {
-	if (consum[i] > 0.0001)
-	  consum[i]=SIconsumption2user(consum[i]);
+  switch (currentmethod) {
+  case CONSUM:
+  case FILL:
+  case TRIP:
+  case PRICEPERLITRE:
+    nummonths=db_get_monthly_data(currentyear,&px2,&fill,&trip,&consum,&priceperlitre);
+
+    for (i=0;i<nummonths;i++) {
+      switch (currentmethod) {
+      case CONSUM:
+	yval=consum;
+	if (curunit.consumeunit != SI) {
+	  if (consum[i] > 0.0001)
+	    consum[i]=SIconsumption2user(consum[i]);
+	}
+	break;
+      case FILL:
+	yval=fill;
+	if (curunit.volumeunit != SI) {
+	  if (fill[i] > 0.0001)
+	    fill[i]=SIvolume2user(fill[i]);
+	}
+	break;
+      case TRIP:
+	yval=trip;
+	if (curunit.lengthunit != SI) {
+	  if (trip[i] > 0.0001)
+	    trip[i]=SIlength2user(trip[i]);
+	}
+	break;
+      case PRICEPERLITRE:
+	yval=priceperlitre;
+	if (curunit.consumeunit != SI) {
+	  if (priceperlitre[i] > 0.0001)
+	    priceperlitre[i]=SIppl2user(priceperlitre[i]);
+	}
+	break;
+      default:
+	yval=consum;
       }
-      break;
-    case FILL:
-      yval=fill;
-      if (curunit.volumeunit != SI) {
-	if (fill[i] > 0.0001)
-	  fill[i]=SIvolume2user(fill[i]);
-      }
-      break;
-    case TRIP:
-      yval=trip;
-      if (curunit.lengthunit != SI) {
-	if (trip[i] > 0.0001)
-	  trip[i]=SIlength2user(trip[i]);
-      }
-      break;
-    case PRICEPERLITRE:
-      yval=priceperlitre;
-      if (curunit.consumeunit != SI) {
-	if (priceperlitre[i] > 0.0001)
-	  priceperlitre[i]=SIppl2user(priceperlitre[i]);
-      }
-      break;
-    default:
-      yval=consum;
+      if (yval[i] > ymax) ymax=yval[i];
     }
-    if (yval[i] > ymax) ymax=yval[i];
+    gtk_plot_data_set_points(dataset, px2, yval, NULL, NULL, nummonths);
+    break;
+  case CONSUMDISTR:
+    numyears=db_get_yearly_consum(&year, &numfills, &minconsum, &avgconsum, &maxconsum);
+    if (currentyear<year[0] || currentyear > year[numyears-1]) {
+      numbins=5;
+      binstart=7.0;
+      binend=12.0;
+    }
+    else {
+      i=currentyear-year[0];
+      binstart=minconsum[i];
+      binend=maxconsum[i];
+      numbins=(int)((binend-binstart)/0.5);
+      if (numbins==0)
+	numbins=1;
+    }
+
+    free(year);
+    free(numfills);
+    free(minconsum);
+    free(avgconsum);
+    free(maxconsum);
+
+    if (curunit.consumeunit!=SI) {
+      bintemp=binend;binend=binstart;binstart=bintemp;
+      binstart=SIconsumption2user(binstart);
+      binend=SIconsumption2user(binend);
+    }
+    bindelta=(binend-binstart)/numbins;
+    px2=(double *)malloc(numbins*sizeof(double));
+    bin=(double *)malloc(numbins*sizeof(double));
+    for (i=0, binc=binstart;i<numbins;i++,binc+=bindelta) {
+      px2[i]=binc;
+      if (curunit.consumeunit==SI)
+	bin[i]=(double)db_get_consum_bin(currentyear, binc, binc+bindelta);
+      else
+	bin[i]=(double)db_get_consum_bin(currentyear, user2SIconsumption(binc+bindelta), user2SIconsumption(binc));
+      if (bin[i] > ymax) ymax=bin[i];
+    }
+
+    /* Bars are centered on the start of the bin, correct this */
+    offset=(px2[numbins-1]-px2[0])/(numbins-1)/2;
+    for (i=0; i<numbins; i++) {
+      px2[i]+=offset;
+    }
+    
+    yval=bin;
+    gtk_plot_data_set_points(dataset, px2, yval, NULL, NULL, numbins);
+    break;
   }
-  gtk_plot_data_set_points(dataset, px2, yval, NULL, NULL, nummonths);
   if (ymax>0.0001)
     gtk_plot_set_yrange(GTK_PLOT(pui->active_plot), 0., 1.1*ymax);
   else {
@@ -186,6 +276,9 @@ plot_monthly_data(AppUIData *pui, enum analysis method)
       else
 	ymax=5.0;
       break;
+    case CONSUMDISTR:
+      ymax=5.0;
+      break;
     default:
       ymax=1000;
     }
@@ -204,13 +297,31 @@ plot_monthly_data(AppUIData *pui, enum analysis method)
   g_snprintf(yearbuf,YEARBUFLEN,"%s %d",_("Year"),currentyear);
   gtk_plot_data_set_legend(dataset, yearbuf);
 
-  gtk_plot_bar_set_width(GTK_PLOT_BAR(dataset), 0.25);
+  switch (currentmethod) {
+  case CONSUM:
+  case FILL:
+  case TRIP:
+  case PRICEPERLITRE:
+    gtk_plot_bar_set_width(GTK_PLOT_BAR(dataset), 0.25);
+    gtk_plot_set_xrange(GTK_PLOT(pui->active_plot), 0. , 13.0);
+    gtk_plot_axis_set_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), pui->months);
+    gtk_plot_axis_use_custom_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), TRUE);
+    break;
+  case CONSUMDISTR:
+    if (numbins>1) {
+      gtk_plot_bar_set_width(GTK_PLOT_BAR(dataset), (px2[numbins-1]-px2[0])/(numbins-1)/2);
+    }
+    else {
+      gtk_plot_bar_set_width(GTK_PLOT_BAR(dataset), SIconsumption2user(0.5));
+    }
+    gtk_plot_set_xrange(GTK_PLOT(pui->active_plot), binstart-bindelta/2.0, binend+bindelta/2.0);
+    gtk_plot_axis_use_custom_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), FALSE);
+    break;
+  }
 
   gtk_plot_canvas_paint(GTK_PLOT_CANVAS(pui->canvas));
   gtk_widget_queue_draw(GTK_WIDGET(pui->canvas));
 
-  gtk_plot_axis_set_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), pui->months);
-  gtk_plot_axis_use_custom_tick_labels(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), TRUE);
 }
 
 void browse_prev_year(AppUIData *pui)
@@ -449,6 +560,8 @@ void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
   pui->sm_item_monthlytrip = gtk_radio_menu_item_new_with_label(radio_menu_group,_("Monthly driving distance"));
   radio_menu_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (pui->sm_item_monthlytrip));
   pui->sm_item_monthlypriceperlitre = gtk_radio_menu_item_new_with_label(radio_menu_group,_("Monthly average fuel price"));
+  radio_menu_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (pui->sm_item_monthlypriceperlitre));
+  pui->sm_item_consumdistr = gtk_radio_menu_item_new_with_label(radio_menu_group,_("Yearly fill distribution"));
 
   /* Create view menu items */
   pui->stat_menu_item_fullscreen = gtk_check_menu_item_new_with_label(_("Full screen"));
@@ -479,6 +592,7 @@ void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
   gtk_menu_append(menu_analysis, pui->sm_item_monthly);
   gtk_menu_append(menu_analysis, pui->sm_item_monthlytrip);
   gtk_menu_append(menu_analysis, pui->sm_item_monthlypriceperlitre);
+  gtk_menu_append(menu_analysis, pui->sm_item_consumdistr);
 
 #if PLAINGTK == 1
   menubar = gtk_menu_bar_new();
@@ -502,6 +616,8 @@ void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
 			   G_CALLBACK(cb_trip), pui);
   g_signal_connect(G_OBJECT(pui->sm_item_monthlypriceperlitre), "toggled",
 			   G_CALLBACK(cb_priceperlitre), pui);
+  g_signal_connect(G_OBJECT(pui->sm_item_consumdistr), "toggled",
+			   G_CALLBACK(cb_consumdistr), pui);
   g_signal_connect(G_OBJECT(item_save), "activate",
 		   G_CALLBACK(callback_save_stat), pui);
 #if PLAINGTK == 0
@@ -581,7 +697,33 @@ void plot_set_titles( AppUIData *pui, enum analysis method)
 				     GTK_JUSTIFY_CENTER,
 				       _("Average fuel price"));
     break;
+  case CONSUMDISTR:
+      gtk_plot_axis_set_major_ticks(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_LEFT), 1.0);
+      gtk_plot_axis_set_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_LEFT), _("Number of fills"));
+    pui->maintitle = gtk_plot_canvas_text_new("Helvetica-Bold", 16, 0, NULL, NULL, TRUE,
+				     GTK_JUSTIFY_CENTER,
+				       _("Yearly fill distribution"));
+      break;
   }
+
+  switch (method) {
+  case CONSUM:
+  case FILL:
+  case TRIP:
+  case PRICEPERLITRE:
+    gtk_plot_axis_set_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), _("Month"));
+    break;
+  case CONSUMDISTR:
+    if (curunit.consumeunit==SI) {
+      gtk_plot_axis_set_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), _("Consumption [l/100km]"));
+    }
+    else {
+      gtk_plot_axis_set_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), _("Miles per gallon"));
+    }
+    break;
+  }
+
+
   gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(pui->canvas), pui->maintitle, .50, YSCALE(.050), .0, .0);
 
 }
@@ -615,7 +757,7 @@ void create_statwin( AppUIData *pui, enum analysis method)
 #if PLAINGTK == 1
   pui->app->statwindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(pui->app->statwindow), "Plot");
-  gtk_widget_set_usize(pui->app->statwindow,800,400);
+  gtk_widget_set_size_request(pui->app->statwindow,800,400);
   gtk_container_border_width(GTK_CONTAINER(pui->app->statwindow),0);
 #else
   pui->app->statwindow = HILDON_WINDOW(hildon_window_new());
@@ -745,6 +887,11 @@ void callback_stat_priceperlitre( GtkAction * action, AppUIData *pui )
   create_statwin(pui, PRICEPERLITRE);
 }
 
+void callback_stat_consumdistr( GtkAction * action, AppUIData *pui )
+{
+  create_statwin(pui, CONSUMDISTR);
+}
+
 void cb_stat( GtkAction * action, AppUIData *pui )
 {
   plot_monthly_data(pui, CONSUM);
@@ -759,3 +906,8 @@ void cb_priceperlitre( GtkAction * action, AppUIData *pui )
 {
   plot_monthly_data(pui, PRICEPERLITRE);
 }
+
+void cb_consumdistr( GtkAction * action, AppUIData *pui )
+{
+  plot_monthly_data(pui, CONSUMDISTR);
+}
